#ifndef _MASTERRD_H_
#define _MASTERRD_H_

/**********************************************************
for MasterRD.dll Version: 5.8.0.2
**********************************************************/

/**********************************************************
    the return value >= 0 is successful
    < 0 is failure, the failure reason in the table
**********************************************************/
#define LIB_SUCCESS        0  // successful
#define LIB_PARAM_ERR     -1  // paremeter error
#define LIB_OS_ERR        -2  // OS error
#define LIB_NETWORK_ERR   -3  // net work error (socket can not assign, can not establish connection)
#define LIB_TIMEOUT       -4  // operation time out
#define LIB_FAILED        -5  // operation failed
#define LIB_BUS_EXIST     -6  // bus already exist
#define LIB_READER_EXIST  -7  // reader already exist

#define MAX_RF_BUFFER     1024

#define _SAM_TIMEOUT_          1500
#define _ISO14443_4_TIMEOUT_   5100
#define _FAST_FUN_TIMEOUT_     100

#define SAM_CMD_INIT_BAUD      0x90
#define SAM_CMD_RST            0x91
#define SAM_CMD_SET_PPS        0x92
#define SAM_CMD_COS            0x93
#define ISO14443A_CMD_ANTICOLLISION_AND_SELECT 0x94

// com port property
typedef struct tagCommPortProperty
{
    UINT nPort; // com port
    UINT nBaud; // baud rate
} CommPortProperty;


// net work property
typedef struct tagTCPPortProperty
{
    char szAddr[16];  // ip address e.g.: 192.168.8.88
    WORD nPort;       // com port number
} TCPPortProperty;


/**********************************************************
    Funciton:   load MasterRD.dll
    Parameter:  phInstMaster is a pointer to dll
    return:     TURE is successful
    Description:    uses at program start to load functions in dll
**********************************************************/
BOOL MasterRD_Load_rfLib(HINSTANCE *phInstMaster);


/**********************************************************
    Funciton:   free dll
    Parameter:  phInstMaster is a pointer to dll
    return:     TURE is successful
    Description:    use at program end before exit
**********************************************************/
BOOL MasterRD_Free_rfLib(HINSTANCE *phInstMaster);


/**********************************************************
    Funciton:   get dll version 
    Parameter:  pnVer: pointer to version out put, 4 bytes
    return:     reference error table
    Description:    
**********************************************************/
extern int    (WINAPI *rf_GetVer)(UINT *pnVer);


/**********************************************************
    Funciton:   DES encryption
    Parameter:  pcOut:   pointer to encryption resulted data output
            pcIn:    pointer to encryption data
            nInLen:  encryption data input length 
            pcKey:   pointer to encryption key 
            nKeyLen: key length
    return:     0 is successful
    Description:    about the key length, 8 bytes for single DES, 16 bytes for triple DES, lack of zeros
**********************************************************/
extern int    (WINAPI *rf_DES_Encrypt)(BYTE *pcOut, BYTE *pcIn, UINT nInLen, BYTE *pcKey, UINT nKeyLen);


/**********************************************************
    Funciton:   DES decryption
    Parameter:  pcOut:   decryption resulted data output pointer
            pcIn:    pointer to decryption data
            nInLen:  decryption data input length 
            pcKey:   decryption key 
            nKeyLen: key length
    return:     0 is successful
    Description:    about the key length, 8 bytes for single DES, 16 bytes for triple DES, lack of zeros
**********************************************************/
extern int    (WINAPI *rf_DES_Decrypt)(BYTE *pcOut, BYTE *pcIn, UINT nInLen, BYTE *pcKey, UINT nKeyLen);


/**********************************************************
    Funciton:   open com port with baud rate
    Parameter:  port: 1 to 256
            baud: 9600 to 115200
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_COM_Open)(UINT port, long baud);


/**********************************************************
    Funciton:   close com port
    Parameter:  port: port number opened
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_COM_Close)(UINT port);


/**********************************************************
    Funciton:   check open status of the com port
    Parameter:  port: port number to check
    return:     opened return TRUE, else return FALSE
    Description:    
**********************************************************/
extern BOOL   (WINAPI *rf_COM_IsOpen)(UINT port);


/**********************************************************
    Funciton:   get baud rate of the opened com port
    Parameter:  port: port number opened
    return:     baud rate
    Description:    
**********************************************************/
extern int    (WINAPI *rf_COM_GetBaud)(UINT port);


/**********************************************************
    Funciton:   open network port
    Parameter:  szIPAddr: IP address, e.g. 192.168.1.2
            port:   port number
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_Net_Open)(LPCSTR szIPAddr, UINT port);


/**********************************************************
    Funciton:   close network port
    Parameter:  szIPAddr: IP address, e.g. 192.168.1.2
            port:   port number
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_Net_Close)(LPCSTR szIPAddr, UINT port);


/**********************************************************
    Funciton:   check the open status of the network port
    Parameter:  szIPAddr: IP address, e.g. 192.168.1.2
            port:   port number
    return:     opened return TRUE, else return FALSE
    Description:    
**********************************************************/
extern int   (WINAPI *rf_Net_IsOpen)(LPCSTR szIPAddr, UINT port);


/**********************************************************
    Funciton:   open reader on newwork port
    Parameter:  szIPAddr: IP address, e.g. 192.168.1.2
            port:   port number
            nReaderID: ID or reader, 2 bytes int, need to macth the ID of the reader
    return:     0 is successful
    Description:    need to open newwork port first, a newwork port could connect more readers with different ID

**********************************************************/
extern int    (WINAPI *rf_Reader_TcpOpen)(LPCSTR szIPAddr, UINT port, WORD nReaderID);



/**********************************************************
    Funciton:   open reader on com port
    Parameter:  port:   the com port number opened
            nReaderID: ID or reader, 2 bytes int, need to macth the ID of the reader
    return:     0 is successful
    Description:    need to open com port first, a com port could connect more readers with different ID


**********************************************************/
extern int    (WINAPI *rf_Reader_ComOpen)(UINT port, WORD nReaderID);


/**********************************************************
    Funciton:   close reader
    Parameter:  nReaderID: the reader ID need to close
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_Reader_Close)(WORD nReaderID);


/**********************************************************
    Funciton:   check the open status of specified ID of the reader on all bus/port of local PC
    Parameter:  nReaderID: the reader ID need to check
    return:     opened return TRUE, else return FALSE
    Description:    
**********************************************************/
extern BOOL    (WINAPI *rf_Reader_IsOpen)(WORD nReaderID);


/*********************************************************
    Funciton:   modify reader ID
    Parameter:  nNewReaderID: new reader ID, 2 bytes int
            nReaderID: old reader ID
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_Reader_SetID)(WORD nReaderID, WORD nNewReaderID);


/**********************************************************
    Funciton:   get current connected reader ID
    Parameter:  pnReaderID: pointer to the reader ID output
    return:     0 is successful
    Description:    this function uses broad cast to get connected reader ID, then user need be sure only 1 reader has connected
**********************************************************/
extern int    (WINAPI *rf_Reader_GetID)(WORD *pnReaderID);


/********************************************************
    Funciton:   get firmware version
    Parameter:  nReaderID: reader ID
            pnVersion: pointer to version of firmware, version is 2 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_Reader_GetFirmwareVer)(WORD nReaderID, WORD *pnVersion);


/*********************************************************
    Funciton:   get reader SNR (8 bytes)
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to reader SNR (8 bytes)
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_Reader_GetSN)(WORD nReaderID, BYTE *pcSNR);

/*********************************************************
    Funciton:   set buzzer beep
    Parameter:  nReaderID: reader ID
            cTmie: beep time, unit is 10ms
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_Reader_SetBeep)(WORD nReaderID, BYTE cTmie);


/*********************************************************
    Funciton:   set Light
    Parameter:  nReaderID: reader ID
            cColor: color of LED to set, 0: dark; 1: red; 2: green; 3: yellow
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_Reader_SetLight)(WORD nReaderID, BYTE cColor);
/*********************************************************
  Funciton: turn on or turn off the reader antenna  //no export
  Parameter: nReaderID: reader ID
       		 model: antenna state
  return: 0 is successful
  Description:  mode=0: turn off antenna
        		mode=1: turn on antenna
*********************************************************/
extern int    (WINAPI *rf_Reader_SetAntenna)(WORD nReaderID, BYTE state);

/*********************************************************
    Funciton:   turn ON RF reader antenna
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_Reader_AntennaON)(WORD nReaderID);


/*********************************************************
    Funciton:   turn OFF RF reader antenna
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_Reader_AntennaOFF)(WORD nReaderID);
// Seek Baudrate ID 
extern BYTE   (WINAPI *rf_SAM_GetBaudID)(UINT baud);
/*********************************************************
    Funciton:   set SAM default baud rate
    Parameter:  nReaderID: reader ID
            cSamNo: physical number of SAM, the value is: 1, 2, 3, 4...
            nBaudRate: baud rate, the value is 9600, 19200, 38400, 55800, 57600, 115200, 230400, determin by supported reader
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_SAM_InitBaudrate)(WORD nReaderID, BYTE cSamNo, UINT nBaudRate);


/*********************************************************
    Funciton:   reset SAM
    Parameter:  nReaderID: reader ID
            cSamNo: physical number of SAM, the value is: 1, 2, 3, 4...
            pcRstInfo: reset infomation of the SAM card
            pcDataLen: reset infomation length
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_SAM_RST)(WORD nReaderID, BYTE cSamNo, BYTE *pcRstInfo, BYTE *pcDataLen);


/*********************************************************
    Funciton:   send APDU to SAM card
    Parameter:  nReaderID: reader ID
            cSamNo:   physical number of SAM, the value is: 1, 2, 3, 4...
            pcApduIn: pointer to APDU to send
            cApduLen: APDU length
            pcResultOut: pointer to returned result from reader, include SW1 SW2
            pcResultLen: pointer to result length
            dwTimeOut: the APDU exchange time out, unit is ms, if input 0, uses default value: 1500ms
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_SAM_COS)(WORD nReaderID, BYTE cSamNo, BYTE *pcApduIn, BYTE cApduLen, BYTE *pcResultOut, BYTE *pcResultLen, DWORD dwTimeOut);


/*********************************************************
    Funciton:   set PPS of SAM card
    Parameter:  nReaderID: reader ID
            cSamNo: physical number of SAM, the value is: 1, 2, 3, 4...
            nBaudRate: target baud rate, the value is 9600, 19200, 38400, 55800, 57600, 115200, 230400, determin by supported SAM card
    return:     0 is successful
    Description:    if this function failure, check the SAM manual to make sure the SAM card support the baud rate please
*********************************************************/
extern int    (WINAPI *rf_SAM_SetPPS)(WORD nReaderID, BYTE cSamNo, UINT nBaudRate);


/*********************************************************
    Funciton:   set reader RF working mode
    Parameter:  nReaderID: reader ID
            cType: working mode; 'A': ISO14443A; 'B': ISO14443B; '1': ISO15693; 's': ST SR series; 'r': AT88RF020
    return:     0 is successful
    Description:    if this function failure, make sure the reader support this working type please
*********************************************************/
extern int    (WINAPI *rf_Reader_SetISOType)(WORD nReaderID, BYTE cType);


/*********************************************************
    Funciton:   ISO14443A get card(request + 3 level anticollsion + select)
    Parameter:  nReaderID: reader ID
            cMode: request Mode; 0x26: REQA; 0x52: WUPA
            pcDataOut: pointer to returned data while sucessful, format is: ATQA(2 Bytes)+SAK(1 Byte)+SNR(4/7/10 Bytes)
            pcDataLen: pointer to returned data length, the value could be 7(SNR is 4 bytes)/10(SNR is 7 bytes)/13(SNR is 10 bytes)
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_GetCard)(WORD nReaderID, BYTE cMode, BYTE *pcDataOut, BYTE *pcDataLen);


/*********************************************************
    Funciton:   ISO14443 request card
    Parameter:  nReaderID: reader ID
            cMode: request Mode; 0x26: REQA(request except halt cards); 0x52: WUPA(request all include halt cards)
            cpATQA: pointer to ATQA, 2 bytes, returned from card
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_Request)(WORD nReaderID, BYTE cMode, USHORT *cpATQA);


/*********************************************************
    Funciton:   3 level anticollsion and select
    Parameter:  nReaderID: reader ID
            pcOutData: pointer to output data, format is: SAK(1 byte) + SNR(4/7/10 bytes)
            pcDataLen: pointer to data output length, the value should be: 5/8/11 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_AnticollisionAndSelect)(WORD nReaderID, BYTE *pcOutData, BYTE *pcDataLen);


/*********************************************************
    Funciton:   level 1 anticollision
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to output SNR(4 bytes)
    return:     0 is successful
    Description:    uses after a successful rf_ISO14443A_Request
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_AnticollisionLevel1)(WORD nReaderID, BYTE *pcSNR);


/*********************************************************
    Funciton:   level 2 anticollision
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to output SNR(4 bytes)
    return:     0 is successful
    Description:    uses after a successful rf_ISO14443A_SelectLevel1
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_AnticollisionLevel2)(WORD nReaderID, BYTE *pcSNR);


/*********************************************************
    Funciton:   level 3 anticollision
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to output SNR(4 bytes)
    return:     0 is successful
    Description:    uses after a successful rf_ISO14443A_SelectLevel2
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_AnticollisionLevel3)(WORD nReaderID, BYTE *pcSNR);


/*********************************************************
    Funciton:   level 1 select
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to SNR input
            pcSAK: pointer to SAK output, 1 byte
    return:     0 is successful
    Description:    uses after a successful rf_ISO14443A_AnticollisionLevel1
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_SelectLevel1)(WORD nReaderID, BYTE *pcSNR, BYTE *pcSAK);


/*********************************************************
    Funciton:   level 1 select
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to SNR input
            pcSAK: pointer to SAK output, 1 byte
    return:     0 is successful
    Description:    uses after a successful rf_ISO14443A_AnticollisionLevel2
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_SelectLevel2)(WORD nReaderID, BYTE *pcSNR, BYTE *pcSAK);


/*********************************************************
    Funciton:   level 1 select
    Parameter:  nReaderID: reader ID
            pcSNR: pointer to SNR input
            pcSAK: pointer to SAK output, 1 byte
    return:     0 is successful
    Description:    uses after a successful rf_ISO14443A_AnticollisionLevel3
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_SelectLevel3)(WORD nReaderID, BYTE *pcSNR, BYTE *pcSAK);


/**********************************************************
    Funciton:   halt current selected card
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_ISO14443A_Halt)(WORD nReaderID);


/**************************************************************
    Funciton:   set authentication standard
    Parameter:  nReaderID: reader ID
            cStandard: target standard; 0x00: NXP standard(default); 0x01: SHANGHAI standard; other value: RFU
    return:     0 is successful
    Description:    for SHANGHAI applications only
***************************************************************/
extern int    (WINAPI *rf_Reader_SetAuthenticationStandard)(WORD nReaderID, BYTE cStandard);


/*********************************************************
    Funciton:   Mifare 1K/4K (S50/S70) authentication by specified key
    Parameter:  nReaderID: reader ID
            cMode: authentication mode; 0x60: authenticate key A; 0x61: authenticat key B
            cBlock: absolute block number to authenticate; 0x00 - 0x3F for Mifare S50; 0x00 - 0xFF for Mifare S70
            pcKey: pointer to specified key
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_M1_Authentication)(WORD nReaderID, BYTE cMode, BYTE cBlock, BYTE *pcKey);


/*************************************************************** 
    Funciton:   download M1 card key to reader
    Parameter:  nReaderID: reader ID
            cKeyIndex: key index to download
            pcKey: pointer to the key to download
    return:     0 is successful
    Description:    
***************************************************************/
extern int    (WINAPI *rf_M1_DownloadKeyToReader )(WORD nReaderID, BYTE cKeyIndex, BYTE *pcKey);


/**********************************************************
    Funciton:   M1 authentication by downloaded key
    Parameter:  nReaderID: reader ID
            cMode: authentication mode; 0x60: authenticate key A; 0x61: authenticat key B
            cBlock: absolute block number to authenticate; 0x00 - 0x3F for Mifare S50; 0x00 - 0xFF for Mifare S70
            cKeyIndex: the index of the key
    return:     0 is successful
    Description:    need use rf_M1_DownloadKeyToReader to download key first
**********************************************************/
extern int    (WINAPI *rf_M1_AuthenticateByDownloadedKey)(WORD nReaderID, BYTE cMode, BYTE cBlock, BYTE cKeyIndex);


/*********************************************************  
    Funciton:   M1 read data from a block (16 bytes)
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number to read; 0x00 - 0x3F for Mifare S50; 0x00 - 0xFF for Mifare S70
            pData:  pointer to output data, the length is 16 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_M1_Read)(WORD nReaderID, BYTE cBlock, BYTE *pData/*, BYTE* pLen*/);

/*********************************************************  
  Funciton: M1 read data from multi blocks // no export
        read multi blocks of mifareone
  Parameter: nReaderID: reader ID
        StartBlock: start block NO.
		BlockCount: read blcok count
        pData:  pointer to output data
        pLen:   data lenth of block
  return: 0 is successful
*********************************************************/

extern int 	  (WINAPI *rf_M1_MultiRead)(WORD nReaderID, BYTE StartBlock, BYTE BlockCount, BYTE *pData, BYTE* pLen);
/********************************************************* 
    Funciton:   M1 write data to a block (16 bytes)
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number to write; 0x00 - 0x3F for Mifare S50; 0x00 - 0xFF for Mifare S70
            pData:  pointer to input data, the length is 16 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_M1_Write)(WORD nReaderID, BYTE cBlock, BYTE *pData/*, BYTE len*/);

/*********************************************************  
  Funciton:  M1 write data to multi block //no export
        read multi blocks of mifareone
  Parameter: nReaderID: reader ID
        StartBlock: start block NO.
		BlockCount: write block count
        pData:      pointer to input data
  return:  0 is successful
*********************************************************/
extern int    (WINAPI *rf_M1_MultiWrite)(WORD nReaderID, BYTE StartBlock, BYTE BlockCount, BYTE *pData);

/********************************************************* 
    Funciton:   create purse
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number to creat purse
            
    return:     0 is successful
    Description:    the sector trailer can't be purse
*********************************************************/
extern int    (WINAPI *rf_M1_PurseCreate)(WORD nReaderID, BYTE cBlock, int iValue);


/*********************************************************
    Funciton:   read purse remain value
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number to read purse
            piValue: pointer to value output
    return:     0 is successful
    Description:    the sector trailer can't be purse
*********************************************************/
extern int    (WINAPI *rf_M1_PurseRead)(WORD nReaderID, BYTE cBlock, int *piValue);


/********************************************************* 
    Funciton:   purse decrement
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number to decrement
            iValue: the decrement value
    return:     0 is successful
    Description:    the sector trailer can't be purse
*********************************************************/
extern int    (WINAPI *rf_M1_PurseDecrement)(WORD nReaderID, BYTE cBlock, int iValue);


/*********************************************************
    Funciton:   purse increment
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number to increment
            iValue: the increment value
    return:     0 is successful
    Description:    the sector trailer can't be purse
*********************************************************/
extern int    (WINAPI *rf_M1_PurseIncrement)(WORD nReaderID, BYTE cBlock, int iValue);





/*********************************************************
    Funciton:   data restore from a block to card buffer
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number
    return:     0 is successful
    Description:    uses before rf_M1_Transfer
*********************************************************/
extern int    (WINAPI *rf_M1_PurseRestore)(WORD nReaderID, BYTE cBlock);


/*********************************************************
    Funciton:   transfer the data in card buffer to a block(eeprom)
    Parameter:  nReaderID: reader ID
            cBlock: absolute block number
    return:     0 is successful
    Description:    uses after rf_M1_Restore
*********************************************************/
extern int    (WINAPI *rf_M1_PurseTransfer)(WORD nReaderID, BYTE cBlock);

/*********************************************************
    Funciton:   backup(copy) a purse from block to another block in same sector
    Parameter:  nReaderID: reader ID
            cSrcBlock: source block, absolute block number
            cDestBlock: destination block, absolute block number
    return:     0 is successful
    Description:    the sector trailer can't be purse, the source and target block MUST in same sector
*********************************************************/
extern int    (WINAPI *rf_M1_PurseBackup)(WORD nReaderID, BYTE cSrcBlock, BYTE cDestBlock);

/*********************************************************
    Funciton:   ultra light card write a block(4 bytes)
    Parameter:  nReaderID: reader ID
            cBlock: block number of ultra light
            pcData: pointer to data to write(4 bytes)
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_UltraLight_Write)(WORD nReaderID, BYTE cBlock, BYTE *pcData);


/*********************************************************
    Funciton:   ultra light card read data(4 blocks)
    Parameter:  cBlock: start block number of ultra light
                pcData: pointer to output data
    return:     0 is successful
    Description:    if start block number is 0x0F, then read 4 blocks: 0x0F, 0x00, 0x01 and 0x02
*********************************************************/
extern int    (WINAPI *rf_UltraLight_Read)(WORD nReaderID, BYTE cBlock, BYTE *pcData/*, BYTE* pcLen*/);

/*********************************************************
    Funciton:   Ultralight C card Authentication
    Parameter:  pcKey: 16 byte Key
    return:     0 is successful
*********************************************************/
extern int    (WINAPI *rf_UltraLightC_Authentication)(WORD nReaderID, BYTE *pcKey);

/********************************************************* 
 Funciton:  ISO14443-4 TypeA search and reset // canceled  no export
 Parameter:  nReaderID:  reader ID
             model:  search card mode(0x26-REQA,0x52-WUPA)
             pData:  return data
             pMsgLg: data length
 return: 0 is successful
*********************************************************/
extern int    (WINAPI *rf_ISO14443A_RST)(WORD nReaderID,BYTE model,BYTE *pData,BYTE* pMsgLg);
/*****************************************************************
    Funciton:   send APDU to ISO14443-4 smart card
    Parameter:  nReaderID: reader ID
            pcApduIn: pointer to APDU to send
            cApduLen: length of APDU input
            pcResultOut: pointer to result from card, include SW1 SW2
            pcResultLen: pointer to result length
            dwTimeOut: the APDU exchange time out, unit is ms, if input 0, uses default value: 5100ms
    return:     0 is successful
    Description:    uses after rf_ISO14443_4_TypeA_RATS and (rf_ISO14443B_Request + rf_ISO14443B_Attribute)
*****************************************************************/
extern int    (WINAPI *rf_ISO14443_4_COS)(WORD nReaderID, BYTE *pcApduIn, BYTE cApduLen, BYTE *pcResultOut, BYTE* pcResultLen, DWORD dwTimeOut);


/********************************************************* 
    Funciton:   ISO14443-4 TYPE A card RATS(request for answer to select)
    Parameter:  nReaderID: reader ID
            pcATS: pointer to ATS(answer to select)
    return:     0 is successful
    Description:    uses after a ISO14443 select command and the SAK indicate this card support ISO14443-4
*********************************************************/
extern int    (WINAPI *rf_ISO14443_4_TypeA_RATS)(WORD nReaderID, BYTE *pcATS, BYTE *pcLen);


/*****************************************************************
    Funciton:   request ISO14443 type B card and set ATTRIB
    Parameter:  nReaderID: reader ID
            cMode: request mode; 0: REQB; 1: WUPB
            pcATQB: pointer to ATQB output(12 bytes)
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_ISO14443B_Request)(WORD nReaderID, BYTE cMode, BYTE *pcATQB/*, BYTE *pDataLen*/);

/*****************************************************************
	Funciton:	request ISO14443 type B card and set ATTRIB
	Parameter:	nReaderID: reader ID
			cMode : request mode; 0: REQB; 1: WUPB
			cAFI  : Application Family Identifier
			cNSlot: Number of slots for anticollision process
			pcATQB: pointer to ATQB output(12 bytes)
	return:		0 is successful
	Description:	
*****************************************************************/
extern int    (WINAPI *rf_ISO14443B_GetCard)(WORD nReaderID, BYTE cMode, BYTE cAFI, BYTE cNSlot, BYTE *pcATQB);

/*****************************************************************
  Funciton:  activate the ISO14443-B card // canceled  no export
  Parameter:  nReaderID: reader ID
        PUPI:  card unique ID
        CID:  soecify the logical addresses of card, belong 0 to 14 and less than slotmax 
  return:  0 is successful
*****************************************************************/
extern int    (WINAPI *rf_ISO14443B_Attribute)(WORD nReaderID, unsigned char *pcPUPI, BYTE CID);
/***************************************************************** 
    Funciton:   halt specified type B card
    Parameter:  nReaderID: reader ID
            dwPUPI: PUPI of the card to halt
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_ISO14443B_Halt)(WORD nReaderID, unsigned char *pcPUPI);


/*****************************************************************
    Funciton:   authentication of AT88RF020(8 bytes key)
    Parameter:  nReaderID: reader ID
            pcKey: pointer to the key, length 8 bytes
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_AT88RF020_Check)(WORD nReaderID, BYTE *pcKey);


/*****************************************************************
    Funciton:   read a page of AT88RF020
    Parameter:  nReaderID: reader ID
            cPageAddr: page number, 0 - 31
            pDataOut:  pointer to data output, 8 bytes
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_AT88RF020_Read)(WORD nReaderID, BYTE cPageAddr, BYTE *pDataOut/*, BYTE* pMsgLen*/);


extern int    (WINAPI *rf_AT88RF020_MultiRead)(WORD nReaderID, BYTE StartBlock, BYTE BlockCount, BYTE *pData, BYTE* pLen);

/*****************************************************************
    Funciton:   write a page of AT88RF020
    Parameter:  nReaderID: reader ID
            cPageAddr: page number, 0 - 31
            pDataIn:  pointer to data input, 8 bytes
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_AT88RF020_write)(WORD nReaderID, BYTE cPageAddr, BYTE *pDataIn/*,BYTE len*/);


extern int    (WINAPI *rf_AT88RF020_MultiWrite)(WORD nReaderID, BYTE StartBlock, BYTE BlockCount, BYTE *pData);

/***************************************************************** 
    Funciton:   write the lock config page of AT88RF020
    Parameter:  nReaderID: reader ID
            pDataIn: pointer to config data, 4 bytes
    return:     0 is successful
    Description:    please read user's manual of AT88RF020 carefully before use this function
*****************************************************************/
extern int    (WINAPI *rf_AT88RF020_Lock)(WORD nReaderID, BYTE *pDataIn/*,BYTE len*/);


/***************************************************************** 
    Funciton:   count of AT88RF020
    Parameter:  nReaderID: reader ID
            pDataIn: pointer count data, 6 bytes
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_AT88RF020_Count)(WORD nReaderID, BYTE *pDataIn/*, BYTE len*/);


/***************************************************************** 
    Funciton:   deselect current AT88RF020
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
*****************************************************************/
extern int    (WINAPI *rf_AT88RF020_Deselect)(WORD nReaderID);


/*********************************************************
    Funciton:   initial and select a ST series card
    Parameter:  nReaderID: reader ID
            pcCardID: pointer to card ID output, 1 byte
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_GetCard)(WORD nReaderID, BYTE *pcCardID);


/*********************************************************
    Funciton:   completion ST series card
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_Completion)(WORD nReaderID);


/*********************************************************
    Funciton:   initial and select a SR176 card
    Parameter:  nReaderID: reader ID
            pcCardID: pointer to card ID output, 1 byte
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SR176_Select)(WORD nReaderID, BYTE* pcCardID);


/*********************************************************
    Funciton:   completion SR176 card
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SR176_Completion)(WORD nReaderID);


/*********************************************************
    Funciton:   read a block of SR176(2 bytes)
    Parameter:  nReaderID: reader ID
            cBlock: block number to read
            pcDataOut: pointer to data output, 2 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SR176_ReadBlock)(WORD nReaderID, BYTE cBlock, BYTE *pcDataOut/*, BYTE *pcDataLen*/);
//read multi blocks of sr176 // 
extern int    (WINAPI *rf_ST_SR176_MultiRead)(WORD nReaderID, BYTE cStartBlock, BYTE cBlockCount, BYTE *pData,BYTE* pLen);

/********************************************************* 
    Funciton:   write a block of SR176(2 bytes)
    Parameter:  nReaderID: reader ID
            pDataIn: pointer to data to write, 2 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SR176_WriteBlock)(WORD nReaderID, BYTE cBlock, BYTE *pcDataIn);
//write multi blocks of sr176 // 
extern int    (WINAPI *rf_ST_SR176_MultiWrite)(WORD nReaderID, BYTE cStartBlock, BYTE cBlockCount, BYTE *data);

/*********************************************************  
    Funciton:   protect block of SR176(write LOCK_REG)
    Parameter:  nReaderID: reader ID
            cLockReg: the value of LOCK_REG to write
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SR176_ProtectBlock)(WORD nReaderID, BYTE cLockReg);


/*********************************************************  
    Funciton:   get protection of SR176(read LOCK_REG)
    Parameter:  nReaderID: reader ID
            cLockReg: pointer to LOCK_REG output, 1 byte
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SR176_GetProtectReg)(WORD nReaderID, BYTE *cLockReg);


/*********************************************************
    Funciton:   initial and select a ST SRI series card
    Parameter:  nReaderID: reader ID
            pcCardID: pointer to card ID output, 1 byte
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SRI_Select)(WORD nReaderID, BYTE *pcCardID);


/*********************************************************
    Funciton:   completion ST SRI series card
    Parameter:  nReaderID: reader ID
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SRI_Completion)(WORD nReaderID);


/*********************************************************
    Funciton:   get UID of ST SRI series card
    Parameter:  nReaderID: reader ID
            pcUID: pointer to UID output, 8 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SRI_GetUID)(WORD nReaderID, BYTE *pcUID/*, BYTE *pLen*/);


/*********************************************************  
    Funciton:   read a block of ST SRI series card(4 bytes)
    Parameter:  nReaderID: reader ID
            cBlock: the block number to read
            pcData: pointer to data output, 4 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SRI_ReadBlock)(WORD nReaderID, BYTE cBlock, BYTE *pcData/*, BYTE *pLen*/);
extern int    (WINAPI *rf_ST_SRI_MultiRead)(WORD nReaderID, BYTE cStartBlock, BYTE cBlockCount, BYTE *pData,BYTE* pLen);

/********************************************************* 
    Funciton:   write a block of ST SRI series card(4 bytes)
    Parameter:  nReaderID: reader ID
            cBlock: the block number to write
            pcData: pointer to data to write, 4 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SRI_WriteBlock)(WORD nReaderID, BYTE cBlock, BYTE *pcData);
extern int    (WINAPI *rf_ST_SRI_MultiWrite)(WORD nReaderID, BYTE cStartBlock, BYTE cBlockCount, BYTE *data);

/*********************************************************
    Funciton:   authenticate of ST SRIX4K(anti-clone function of SRIX4K)
    Parameter:  nReaderID: reader ID
            pcRND: pointer to RND input, 6 bytes
            pcSIG: pointer to signature output, 3 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ST_SRIX_Authenticate)(WORD nReaderID, BYTE *pcRND, BYTE *pcSIG/*, BYTE *pLen*/);


/********************************************************** 
    Funciton:   write LOCK_REG of ST SRI series cards
    Parameter:  nReaderID: reader ID
            cLockReg: value of LOCK_REG to write
    return:     0 is successful
    Description:    
**********************************************************/
extern int    (WINAPI *rf_ST_SRI_WriteLockReg)(WORD nReaderID, BYTE cLockReg);



/********************************************************* 
   Funciton:   ISO15693_Inventory
    Parameter:  nReaderID: reader ID
            	    pData: return data
					pLen: pData length
    return:     0 is successful
    Description:   

  
*********************************************************/
extern int    (WINAPI *rf_ISO15693_Inventory)(WORD nReaderID, BYTE *pData, BYTE *pLen);

/*********************************************************

  
    Funciton:  search multi card 
    Parameter:  nReaderID: reader ID
    			cAFIEnable: AFI enable
				AFI: AFI
				pData: return data (1 byte DSFID + 8 byte UID) * N
				pLen: pData length
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO15693_MultiInventory)(WORD nReaderID, BYTE cAFIEnable, BYTE AFI, BYTE *pData, BYTE *pLen);

/*********************************************************
 
  
    Funciton:   ISO15693 stay quiet
    Parameter:  nReaderID: reader ID
            	UID: unique ID 8 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO15693_StayQuiet)(WORD nReaderID, BYTE *UID);

/*********************************************************

  
    Funciton:   ISO15693 select
    Parameter:  nReaderID: reader ID
            	UID: unique ID 8 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO15693_Select)(WORD nReaderID, BYTE *UID);

/********************************************************* 

  
    Funciton:   ISO15693 reset to ready
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO15693_ResetToReady)(WORD nReaderID, BYTE mode, BYTE *UID);

/*********************************************************

  
    Funciton:   ISO15693 read
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				block: block number
				number: output block counts, less than 0x10
				pData: return data
				pLen: pData length
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_Read)(WORD nReaderID, BYTE  model, BYTE  *UID, BYTE  block, BYTE  number, BYTE  *pData, BYTE  *pLen);

/*********************************************************

  
    Funciton:   ISO15693 write
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				block: block number
				data: input data, 4 bytes
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_Write)(WORD nReaderID, BYTE  model, BYTE  *UID, BYTE  block, BYTE *data);

/********************************************************* 

  
    Funciton:   ISO15693 lock block
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				block: block number
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_LockBlock)(WORD nReaderID, BYTE model, BYTE *UID, BYTE block);

/********************************************************* 

  
    Funciton:   ISO15693 write AFI
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				AFI: input AFI
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_WriteAFI)(WORD nReaderID, BYTE model, BYTE *UID, BYTE AFI);

/********************************************************* 

  
    Funciton:   ISO15693 lock AFI
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_LockAFI)(WORD nReaderID, BYTE model, BYTE *UID);

/*********************************************************

  
    Funciton:   ISO15693 write DSFID
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				DSFID: input DSFID
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_WriteDSFID)(WORD nReaderID, BYTE model, BYTE *UID, BYTE DSFID);

/********************************************************* 
  
    Funciton:   ISO15693 lock DSFID
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
    return:     0 is successful
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_LockDSFID)(WORD nReaderID, BYTE model, BYTE *UID);

/********************************************************* 

  
    Funciton:   ISO15693 get system information
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				pData: return data
				pLen: pData length
    return:     0 is successful 
    Description:    
/*********************************************************/
extern int    (WINAPI *rf_ISO15693_GetSystemInfomation)(WORD nReaderID, BYTE  model, BYTE  *UID, BYTE  *pData, BYTE  *pLen);
                          
/********************************************************* 

  
    Funciton:   ISO15693 get block security
    Parameter:  nReaderID: reader ID
            	model: bit0=Select_flags,bit1=Addres_flags
				UID: unique ID 8 bytes
				block: block number
				number: output block counts, less than 0x40
				pData: return data
				pLen: pData length
    return:     0 is successful
    Description:    
*********************************************************/
extern int    (WINAPI *rf_ISO15693_GetBlockSecurity)(WORD nReaderID, BYTE  model, BYTE  *UID, BYTE  block, BYTE  number, BYTE  *pData, BYTE  *pLen);

/********************************************************************

  
    Funciton:   display data on reader
    Parameter:  nReaderID: reader ID
            	cDecodeModel: display decode mode
				cDpPosition: when cDecodeModel is 1, the position of the decimal point, from 0 to 7
				cDisplayData: display data
    return:     0 is successful
				1 is fail
    Description:    cDecodeModel=0: display data is 7 segment signal of LED, couldn't need decode
					cDecodeModel=1: each one of the input data of 0~9, decode 7 segment display by program, if input data is
									10(0x0A), display all off
					cDisplayData: 8 bytes display data
					the decode table of display when cDecodeModel = 0:
					display			decode number
					   '0'       		0x7E
       				   '1'        		0x30
      				   '2'        		0x6D
     				   '3'        		0x79
       				   '4'        		0x33
       				   '5'        		0x5B
        			   '6'        		0x5F
        			   '7'        		0x70
        			   '8'        		0x7F
        			   '9'        		0x7B
********************************************************************/
extern int    (WINAPI *rf_Reader_DigtalDisplay)(WORD nReaderID, BYTE cDecodeModel, BYTE cDpPosition, BYTE * DisplayData);

/********************************************************************

  
    Funciton:  to write data into the FLASH memory of reader 
    Parameter:  nReaderID: reader ID
            	iBlock: written to the FLASH block number
				cStartAddress: starting address within the block written to FLASH
				cBytes: counts of bytes written to FLASH
				writeData: input data
    return:     0 is successful
				1 is failed
    Description:   a single write bytes cannot be more than 224 bytes
				   limiting conditions: cStartAddress + cBytes <= 256 
********************************************************************/
extern int    (WINAPI *rf_Reader_FlashWrite)(WORD nReaderID, UINT iBlock, BYTE cStartAddress, BYTE cBytes, BYTE * WriteData);

/********************************************************************

  
    Funciton:  read data from FLASH memory 
    Parameter:  nReaderID: reader ID
            	iBlock: read FLASH block number
				cStartAddress: starting address within the block read from FLASH
				cBytes: counts of bytes read from FLASH
				pData: return read data
    return:     0 is successful
				1 is fail
    Description:    
********************************************************************/
extern int    (WINAPI *rf_Reader_FlashRead)(WORD nReaderID, UINT iBlock, BYTE cStartAddress, BYTE cBytes, BYTE * pData);

/******** Function: search AT88RF080 card *************************************

  
    Funciton:   search AT88RF080 card
    Parameter:  nReaderID: reader ID
				cAFI: AFI
				cParameter: Parameter
				pData: return data
				pLen: return counts of bytes
    return:     0 is successful
				1 is fail
    Description:    
/********************************************************************/
extern int    (WINAPI *rf_AT88RF080_Request)(WORD nReaderID, BYTE cAFI, BYTE cParameter, BYTE *pData, BYTE *pLen);

/**********************************************************

  
    Funciton:   Level0 write perso
    Parameter:  nReaderID: reader ID
            	block: block address(2Byte)
				data: input data(16Byte)
    return:     0 is successful
    Description:   this function only can be used in Security Level0 
**********************************************************/
extern int    (WINAPI *rf_MFP_L0WritePerso)(WORD nReaderID,USHORT block,BYTE *data);

/**********************************************************

  
    Funciton:   Level0 commit perso
    Parameter:  nReaderID: reader ID          
    return:     0 is successful
    Description:    1, this function only can be used in Security Level0
					2, Mifare Plus switch from Level0 to Level1 or Level3 
					3, Before using this instruction,you need to initialize all AES Key and blocks in Level0,
					only Write Perso instruction is completed can this instruction be excuted
**********************************************************/
extern int    (WINAPI *rf_MFP_L0CommitPerso)(WORD nReaderID);

/**********************************************************

  
    Funciton:   Mifare Plus switch to Level2 or Level3
    Parameter:  nReaderID: reader ID
            	cLevel: mifareplus in Level1 or Level2
				pcKey: switch pcKey of Level1(16Byte)
    return:     0 is successful
    Description:    this function only can be used in Security Level1 or Level2
**********************************************************/
extern int    (WINAPI *rf_MFP_SwitchToHighLevel)(WORD nReaderID, BYTE cLevel,BYTE *pcKey);

/**********************************************************
  Funciton:  Mifare Plus authorize block of data(Not including other key and configuration block)
  Parameter:  nReaderID:    reader ID
        keytype:  key mode('A' or 'B')
        blockid:  absolutely block number of data
        keydata:  key(16Byte)
  return:  0 is successful
  Description:  
        1,this function only can be used in Security Level3
**********************************************************/
extern int    (WINAPI *rf_MFP_DataBlockAuthentication)(WORD nReaderID,BYTE cKeyType, WORD nBlock,BYTE *pcKey);

/**********************************************************
  Funciton:  Mifare Plus read block
  Parameter:  nReaderID:   reader ID
         nBlock:      starting block number
         BlockCount:  read blocks from 1 to 3
         pData:        data of block
  return:  0 is successful
  Description:  
		1,this function only can be used in Security Level3
**********************************************************/
extern int    (WINAPI *rf_MFP_L3Read)(WORD nReaderID, WORD nBlock, BYTE BlockCount,BYTE *pData);

/**********************************************************
  Funciton:  Mifare Plus write block
  Parameter:  nReaderID:     reader ID
         nBlock:  starting block number
		 BlockCount:    write blocks from 1 to 3
		 pData:         data of block
  return:  0 is successful
  Description:  
		1,this function only can be used in Security Level3
**********************************************************/           

extern int    (WINAPI *rf_MFP_L3Write)(WORD nReaderID, BYTE nBlock, BYTE BlockCount, BYTE *pData);

/**********************************************************

  
    Funciton:   Mifare Plus initialize the purse
    Parameter:  nReaderID: reader ID
            	blockid: block number
				purseval:  the original purse value
    return:     0 is successful
    Description:    this function only can be used in Security Level3
**********************************************************/
extern int    (WINAPI *rf_MFP_L3PurseCreate)(WORD nReaderID, WORD nBlock, UINT nPurseValue);

/********************************************************** 
    Funciton:   Mifare Plus read purse
    Parameter:  nReaderID: reader ID
            	nBlock: block number
				pPurseValue: purse value
    return:     0 is successful
    Description:   this function only can be used in Security Level3 
**********************************************************/
extern int    (WINAPI *rf_MFP_L3PurseRead)(WORD nReaderID, WORD nBlock, UINT *pPurseValue);

/**********************************************************

  
    Funciton:   Mifare Plus Top-up
    Parameter:  nReaderID: reader ID
            	nBlock: block number
				nPurseInc: purse increment value
    return:     0 is successful
    Description:    this function only can be used in Security Level3
**********************************************************/
extern int    (WINAPI *rf_MFP_L3PurseIncrement)(WORD nReaderID, WORD nBlock, UINT nPurseInc);

/**********************************************************

  
    Funciton:   Mifare Plus decrement value
    Parameter:  nReaderID: reader ID
            	nBlock: block number
				nPurseDec: pures decrement value
    return:     0 is successful
    Description:  this function only can be used in Security Level3  
**********************************************************/
extern int    (WINAPI *rf_MFP_L3PurseDecrement)(WORD nReaderID, WORD nBlock, UINT nPurseDec);

/**********************************************************

  
    Funciton:   purse backup
    Parameter:  nReaderID: reader ID
            	nSrcBlock: backup the purse block
				nDesBlock:	destination purse block
    return:     0 is successful
    Description:  this function only can be used in Security Level3   
**********************************************************/
extern int    (WINAPI *rf_MFP_L3PurseBackup)(WORD nReaderID, WORD nSrcBlock, WORD nDesBlock);

/**********************************************************

  
    Funciton:   Mifare Plus first authentication
    Parameter:  nReaderID: reader ID
            	AES_keyaddr: authenticated AES key block
				AES_keydata: AES key data
    return:     0 is successful
    Description:   this function only can be used in Security Level1/3 
**********************************************************/
extern int    (WINAPI *rf_MFP_FirstAuthentication)(WORD nReaderID, WORD AES_keyaddr, BYTE *AES_keydata);

/**********************************************************

  
    Funciton:   Mifare Plus second authentication
    Parameter:  nReaderID: reader ID
            	AES_keyaddr: authenticated AES key block 
				AES_keydata: AES key data
    return:     0 is successful
    Description:   this function only can be used in Security Level1/3 
**********************************************************/
extern int    (WINAPI *rf_MFP_FollowingAuthentication)(WORD nReaderID, WORD AES_keyaddr, BYTE *AES_keydata);





/***************************************************************

  
    Funciton:  read id card information (card id and reset information)   
    Parameter:  nReaderID: reader ID
            	cardinfo: return information
				infolen: return information length
    return:     0 is successful
    Description:    
***************************************************************/
extern int    (WINAPI *rf_IDCard_Request)(WORD nReaderID, BYTE *cardinfo, UINT *infolen);

/***************************************************************

  
    Funciton:   send APDU to id card
    Parameter:  nReaderID: reader ID
            	command: APDU instruction
				cmdLen: APDU length
				pData: return information
				Length: return information length
    return:     0 is successful
    Description:    
***************************************************************/
extern int    (WINAPI *rf_IDCard_COS)(WORD nReaderID, BYTE *command, UINT cmdLen, BYTE *pData, UINT *Length);
/*********************************************************
  Funciton:search ICODE1 card
  Parameter:nReaderID: reader ID
        pcDataOut:   ICODE1 card number, 8 bytes
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_ICODE1_Request)(WORD nReaderID,  BYTE *pcDataOut);

/*********************************************************
  Funciton:ICODE1 read card
  Parameter:nReaderID: reader ID
		cBlock   :  read block
        pcDataOut:  data cache 4 bytes
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_ICODE1_Read)(WORD nReaderID,  BYTE cBlock, BYTE *pcDataOut);

/*********************************************************
  Funciton:ICODE1 write card
  Parameter:nReaderID: reader ID
		cBlock   :  write block
        pcDataIn :  write data 4 bytes
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_ICODE1_Write)(WORD nReaderID,  BYTE cBlock, BYTE *pcDataIn);

/*********************************************************
  Funciton:ICODE1 halt
  Parameter:nReaderID: reader ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_ICODE1_Halt)(WORD nReaderID);

/*********************************************************
  Funciton:Desfire Authenticate
  Parameter:nReaderID: reader ID
		ucKeyNum : key number
		pucKey   : key 16 bytes
		pucSessionKey: 16 bytes, the process keys of return
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_Authenticat)(WORD nReaderID, BYTE ucKeyNum, BYTE *pucKey, BYTE *pucSessionKey);


/*********************************************************
  Funciton:Desfire ChangeKeySettings
  Parameter:nReaderID: reader ID
		pucKeySettings   : new setting
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_ChangeKeySettings)(WORD nReaderID, BYTE *pucKeySettings);

/*********************************************************
  Funciton:Desfire GetKeySetting
  Parameter:nReaderID: reader ID
		pucKeySettings: KeySettings (2 bytes);
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetKeySetting)(WORD nReaderID, BYTE *pucKeySettings);

/*********************************************************
  Funciton:Desfire ChangeKey
  Parameter:nReaderID: reader ID
		pucKey   : new Key, 24 bytes
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_ChangeKey)(WORD nReaderID, BYTE ucKeyNum, BYTE *pucKey);

/*********************************************************
  Funciton:Desfire GetKeyVer
  Parameter:nReaderID: reader ID
		pucKeyVer: KeyVer (1 byte)
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetKeyVer)(WORD nReaderID, BYTE ucKeyNo, BYTE *pucKeyVer);

/*********************************************************
  Funciton:Desfire CreateApp
  Parameter:nReaderID: reader ID
        nAID     : Application ID(<= 0xFFFFFF)
		ucKeySettings: key setting
		ucNumOfKeys:  number of keys
		pucKeyVer: KeyVer (1 byte)
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CreateApp)(WORD nReaderID, UINT nAID, BYTE ucKeySettings, BYTE ucNumOfKeys);

/*********************************************************
  Funciton:Desfire DeletApp
  Parameter:nReaderID: reader ID
        nAID     : Application ID(<= 0xFFFFFF)
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_DeletApp)(WORD nReaderID, UINT nAID);

/*********************************************************
  Funciton:Desfire Get Application IDs
  Parameter:nReaderID: reader ID
		pucAids  : Application IDs
		pucAidNum: number of ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetAppIDs)(WORD nReaderID, BYTE *pucAids, BYTE *pucAidNum);

/*********************************************************
  Funciton:Desfire Select Application
  Parameter:nReaderID: reader ID
        nAID     : Application ID(<= 0xFFFFFF)
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_SelectApp)(WORD nReaderID, UINT nAID);

/*********************************************************
  Funciton:Desfire Format PICC
  Parameter:nReaderID: reader ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_FormatPICC)(WORD nReaderID);
/*********************************************************
  Funciton:Desfire Get Version
  Parameter:nReaderID: reader ID
		pucVerInfo: chip data, length is 28 bytes 
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetVer)(WORD nReaderID, BYTE *pucVerInfo);

/*********************************************************
  Funciton:Desfire  Get File IDs
  Parameter:nReaderID: reader ID
		pucFileIDs  : File IDs
		pucFileIDNum: number of ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetFileIDs)(WORD nReaderID, BYTE *pucFileIDs, BYTE *pucFileIDNum);

/*********************************************************
  Funciton:Desfire  Get File Settings
  Parameter:nReaderID: reader ID
		pucFileIDs  : File Settings
		pucLen: data length
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetFileSettings)(WORD nReaderID, BYTE ucFileNo, BYTE *pucFileSettings, BYTE *pucLen);

/*********************************************************
  Funciton:Desfire ChangeFileSettings
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		pucKeySettings   : new setting
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_ChangeFileSettings)(WORD nReaderID, BYTE ucFileNo, BYTE *pucKeySettings, BYTE ucLen);

/*********************************************************
  Funciton:Desfire Create Std DataFile
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		ucComSet : comunication setting
		pucAccessrights: operation permissions
		nFileLen : file length
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CreateStdDataFile)(WORD nReaderID, BYTE ucFileNo, BYTE ucComSet, BYTE *pucAccessRights, UINT32 nFileLen);

/*********************************************************
  Funciton:Desfire Create Backup DataFile
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		ucComSet : comunication setting
		pucAccessrights: operation permissions
		nFileLen : file length
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CreateBackupDataFile)(WORD nReaderID, BYTE ucFileNo, BYTE ucComSet, BYTE *pucAccessRights, UINT32 nFileLen);

/*********************************************************
  Funciton:Desfire CreateValueFile
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		ucComSet : comunication setting
		pucAccessrights: operation permissions
		nLowLimit: minimum value
		nUpperLimit:  maximum value
		nValue   : value
		ucEnableCreditLimit: whether limit the memory
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CreateValueFile)(WORD nReaderID, BYTE ucFileNo, BYTE ucComSet, BYTE *pucAccessRights, UINT32 nLowLimit, UINT32 nUpperLimit, UINT32 nValue, BYTE ucEnableCreditLimit);

/*********************************************************
  Funciton:Desfire CreateLinearRecordFile
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		ucComSet : comunication setting
		pucAccessrights: operation permissions
		nRecordSize:  size of record
		nMaxNumOfRecord:  the largest number of record
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CreateLinearRecordFile)(WORD nReaderID, BYTE ucFileNo, BYTE ucComSet, BYTE *pucAccessRights, UINT32 nRecordSize, UINT32 nMaxNumOfRecord);

/*********************************************************
  Funciton:Desfire CreateCyclicRecordFile
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		ucComSet : comunication setting
		pucAccessrights: operation permissions
		nRecordSize: size of record
		nMaxNumOfRecord: the largest number of record
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CreateCyclicRecordFile)(WORD nReaderID, BYTE ucFileNo, BYTE ucComSet, BYTE *pucAccessRights, UINT32 nRecordSize, UINT32 nMaxNumOfRecord);

/*********************************************************
  Funciton:Desfire DeleteFile
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_DeleteFile)(WORD nReaderID, BYTE ucFileNo);

/*********************************************************
  Funciton:Desfire  ReadData
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiOffset : offset address
		uiLen    : data length
		pucDataOut: output data
		plDataLen: output data length
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_ReadData)(WORD nReaderID, BYTE ucFileNo, UINT uiOffset, UINT uiLen, BYTE *pucDataOut, LONG *plDataLen);

/*********************************************************
  Funciton:Desfire WriteData
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiOffset : offset address
		uiLen    : data length
		pucDataIn: input data
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_WriteData)(WORD nReaderID, BYTE ucFileNo, UINT uiOffset, UINT uiLen, BYTE *pucDataIn);

/*********************************************************
  Funciton:Desfire  Get Value
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		puiValue : data of return
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_GetValue)(WORD nReaderID, BYTE ucFileNo, UINT *puiValue);

/*********************************************************
  Funciton:Desfire  Credit
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiValue  : value
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_Credit)(WORD nReaderID, BYTE ucFileNo, UINT uiValue);

/*********************************************************
  Funciton:Desfire  Debit
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiValue  : value
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_Debit)(WORD nReaderID, BYTE ucFileNo, UINT uiValue);

/*********************************************************
  Funciton:Desfire  LimitedCredit
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiValue  : value
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_LimitedCredit)(WORD nReaderID, BYTE ucFileNo, UINT uiValue);

/*********************************************************
  Funciton:Desfire  WriteRecord
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiOffset : starting record
		uiLen    : record length
		pucDataIn: data
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_WriteRecord)(WORD nReaderID, BYTE ucFileNo, UINT uiOffset, UINT uiLen, BYTE *pucDataIn);

/*********************************************************
  Funciton:Desfire  WriteRecord
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
		uiOffset : starting record
		uiCount  : counts of record
		pucDataOut: data of read
		puiDataLen: data length
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_ReadRecord)(WORD nReaderID, BYTE ucFileNo, UINT uiOffset, UINT uiCount, BYTE *pucDataOut, UINT *puiDataLen);

/*********************************************************
  Funciton:Desfire  Clear Record File
  Parameter:nReaderID: reader ID
		ucFileNo : file ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_ClearRecordFile)(WORD nReaderID, BYTE ucFileNo);

/*********************************************************
  Funciton:Desfire  CommitTransaction
  Parameter:nReaderID: reader ID
  return:0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_CommitTransaction)(WORD nReaderID);

/*********************************************************
  Funciton:Desfire  AbortTransaction
  Parameter:nReaderID: reader ID
  return: 0 is successful
*********************************************************/
extern int    (WINAPI *rf_DF_AbortTransaction)(WORD nReaderID);
#endif